"""RFD API."""

from math import ceil
import requests
from bs4 import BeautifulSoup

try:
    from urllib.parse import urlparse  # python 3
except ImportError:
    from urlparse import urlparse  # python 2


def build_web_path(slug):
    return "https://forums.redflagdeals.com{}".format(slug)


def extract_post_id(url):
    return url.split('/')[3].split('-')[-1]


def is_int(number):
    try:
        int(number)
        return True
    except ValueError:
        return False


def calculate_score(post):
    """Calculate either topic or post score. If votes cannot be retrieved, the score is 0.

    Arguments:
        post {dict} -- pass in the topic/post object

    Returns:
        int -- score
    """
    score = 0
    try:
        score = int(post.get('votes').get('total_up')) - \
            int(post.get('votes').get('total_down'))
    except AttributeError:
        pass

    return score


def get_safe_per_page(limit):
    if limit < 5:
        return 5
    elif limit > 40:
        return 40
    return limit


def users_to_dict(users):
    users_dict = {}
    for user in users:
        users_dict[user.get('user_id')] = user.get('username')
    return users_dict


def strip_html(text):
    return BeautifulSoup(text, "html.parser").get_text()


def is_valid_url(url):
    result = urlparse(url)
    return all([result.scheme, result.netloc, result.path])


def get_threads(forum_id, limit):
    threads = []
    response = requests.get(
        "https://forums.redflagdeals.com/api/topics?forum_id={}&per_page={}".format(forum_id, get_safe_per_page(limit)))
    for topic in response.json().get('topics'):
        threads.append({
            'title': topic.get('title'),
            'score': calculate_score(topic),
            'url': build_web_path(topic.get('web_path')),
        })
    return threads[:limit]


def get_posts(post, count=5, tail=False, per_page=40):
    """Retrieve posts from a thread.

    Args:
        post (str): either post id or full url
        count (int, optional): Description

    Yields:
        list(dict): body, score, and user
    """
    if is_valid_url(post):
        post_id = extract_post_id(post)
    elif is_int(post):
        post_id = post
    else:
        raise ValueError()

    response = requests.get(
        "https://forums.redflagdeals.com/api/topics/{}/posts?per_page=40&page=1".format(post_id))
    total_posts = response.json().get('pager').get('total')
    total_pages = response.json().get('pager').get('total_pages')

    if count == 0:
        pages = total_pages
    if count > per_page:
        if count > total_posts:
            count = total_posts
        pages = ceil(count / per_page)
    else:
        if tail:
            pages = total_pages
        else:
            pages = 1

    if tail:
        start_page = ceil((total_posts + 1 - count) / per_page)
        start_post = (total_posts + 1 - count) % per_page
        if start_post == 0:
            start_post = per_page
    else:
        start_page, start_post = 0, 0

    # Go through as many pages as necessary
    for page in range(start_page, pages + 1):
        response = requests.get(
            "https://forums.redflagdeals.com/api/topics/{}/posts?per_page={}&page={}".format(post_id,
                                                                                             get_safe_per_page(
                                                                                                 per_page),
                                                                                             page))

        users = users_to_dict(response.json().get('users'))

        _posts = response.json().get('posts')

        # Determine which post to start with (for --tail)
        if page == start_page and not start_post == 0:
            if tail:
                _posts = _posts[start_post - 1:]
            else:
                _posts = _posts[:start_post]

        for _post in _posts:
            count -= 1
            if count < 0:
                return
            # Sometimes votes is null
            if _post.get('votes') is not None:
                calculated_score = calculate_score(_post)
            else:
                calculated_score = 0
            yield{
                'body': strip_html(_post.get('body')),
                'score': calculated_score,
                'user': users[_post.get('author_id')],
            }



from __future__ import unicode_literals


import sys
import os
import click
from colorama import init, Fore, Style
from rfd.api import get_threads, get_posts
from rfd.__version__ import __version__

init()
print()


def get_version():
    return 'rfd ' + __version__


def get_terminal_width():
    _, columns = os.popen('stty size', 'r').read().split()
    return int(columns)


def get_vote_color(score):
    if score > 0:
        return Fore.GREEN + " [+" + str(score) + "] "
    elif score < 0:
        return Fore.RED + " [" + str(score) + "] "
    return Fore.BLUE + " [" + str(score) + "] "


@click.group(invoke_without_command=True)
@click.option('--version/--no-version', default=False)
@click.pass_context
def cli(ctx, version):
    """Welcome to the RFD CLI. (RedFlagDeals.com)"""
    if version:
        click.echo(get_version())
    elif not ctx.invoked_subcommand:
        click.echo(ctx.get_help())


@cli.command('version')
def display_version():
    click.echo(get_version())


@cli.command(short_help="Displays posts in a specific thread.")
@click.option('--head', default=0, help='Number of topics. Default is 0, for all topics')
@click.option('--tail', default=0, help='Number of topics. Default is disabled. This will override head.')
@click.argument('post_id')
def posts(post_id, head, tail):
    """Displays posts in a specific thread.

    post_id can be a full url or post id only

    Example:

    \b
    url: https://forums.redflagdeals.com/koodo-targeted-public-mobile-12-120-koodo-5gb-40-no-referrals-2173603
    post_id: 2173603
    """
    if head < 0:
        click.echo("Invalid head.")
        sys.exit(1)

    if tail < 0:
        click.echo("Invalid tail.")
        sys.exit(1)

    # Tail overrides head
    if tail > 0:
        count = tail
    else:
        count = head

    try:
        click.echo("-" * get_terminal_width())
        for post in get_posts(post=post_id, count=count, tail=tail > 0):
            click.echo(" -" + get_vote_color(post.get('score')) + Fore.RESET +
                       post.get('body') + Fore.YELLOW + " ({})".format(post.get('user')))
            click.echo(Style.RESET_ALL)
            click.echo("-" * get_terminal_width())
    except ValueError:
        click.echo("Invalid post id.")
        sys.exit(1)
    except AttributeError:
        click.echo("AttributeError: RFD API did not return expected data.")


@cli.command(short_help="Displays threads in the specified forum.")
@click.option('--count', default=10, help='Number of topics.')
@click.argument('forum_id')
def threads(count, forum_id):
    """Displays threads in the specified forum id.

    Popular forum ids:

    \b
    9 \t hot deals
    14 \t computer and electronics
    15 \t offtopic
    17 \t entertainment
    18 \t food and drink
    40 \t automotive
    53 \t home and garden
    67 \t fashion and apparel
    74 \t shopping discussion
    88 \t cell phones
    """
    _threads = get_threads(forum_id, count)
    for i, thread in enumerate(_threads, 1):
        click.echo(" " + str(i) + "." +
                   get_vote_color(thread.get('score')) + Fore.RESET + thread.get('title'))
        click.echo(Fore.BLUE + " {}".format(thread.get('url')))
        click.echo(Style.RESET_ALL)


if __name__ == '__main__':
    cli()



import io  # for python2
from os import path
from setuptools import setup
try:  # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:  # for pip <= 9.0.3
    from pip.req import parse_requirements
from version import __version__ as version

WORKING_DIR = path.abspath(path.dirname(__file__))

# Get long description from reStructuredTExt
with io.open(path.join(WORKING_DIR, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# read requirements.txt and load into list
REQUIREMENTS_TXT = parse_requirements(
    path.join(WORKING_DIR, "requirements.txt"), session='my_session')
REQUIREMENTS = [str(r.req) for r in REQUIREMENTS_TXT]

setup(
    author='Dave Gallant',
    author_email='davegallant@gmail.com',
    description='CLI for RedFlagDeals.com',
    entry_points={
        'console_scripts': [
            'rfd = rfd.rfd_cli:cli',
        ],
    },
    install_requires=REQUIREMENTS,
    keywords='cli redflagdeals',
    license='Apache License, Version 2.0',
    long_description=long_description,
    name='rfd',
    packages=['rfd'],
    url='https://github.com/davegallant/rfd_cli',
    version=version,
)
